@@ -358,14 +358,22 @@ impl Server {
             .spawn(move || {
                 loop {
                     trace!("listening...");
-                    let pipe_connection = match listener.accept(&listener_quit_flag) {
+                    if listener_quit_flag.load(Ordering::SeqCst) {
+                        info!("listener shutdown for quit flag");
+                        break;
+                    }
+                    let pipe_connection = match listener.accept() {
                         Ok(None) => {
                             continue;
                         }
                         Ok(Some(conn)) => Arc::new(conn),
+                        Err(e) if e.kind() == std::io::ErrorKind::Interrupted => {
+                            error!("got interruption {:?}.  Continue...", e);
+                            continue;
+                        }
                         Err(e) => {
                             error!("listener accept got {:?}", e);
-                            break;
+                            continue;
                         }
                     };
 
